#ifndef __MODELDEF_H
#define __MODELDEF_H

#include "config.h"  /* define NN, Nw, Nq, etc. */

/* please define minimum NN, Nw or Nq in config.h */
/* define the default maximum sites */
#ifndef NN
#define NN CM(64,32,32)  /* default maximum 64 sites (64bit long for JJ) */
#endif
/* define the default maximum number of bi-site-exchanges */
#ifndef Nw
#define Nw (4*(NN))  /* default maximum 4 bonds per site */
#endif
/* define the default maximum number of quad-site-exchanges */
#ifndef Nq
#define Nq (4*(NN))  /* default maximum 4 quad-bonds per site */
#endif
#ifndef CM /* Heisenberg-, t-J- or Hubbard-model coding */  
#define CM(JJ,tJ,tU) JJ   /* if not defined in config.h default=JJ */
#endif

#ifndef NOSZSYM      /* set in config.h or outside 2018.01 */
#define NOSZSYM  0   /* set 0 (default) or 1 (anisotropy breaking Sz) */
#endif

#ifndef HFIELD      /* set in config.h or outside 2018.01, old def+undef */
#define HFIELD  0   /* 0 or 1: using FIELD-terms (hx,hy,hz)*(Sx,Sy,Sz)_i */
#endif


extern double pos[NN][3];               /* site-position = (x,y,z)      */
extern double edg[3][3];                /* edge vectors default=(0,0,0) */
extern int  iw[Nw][2];                  /* array bonds          */
extern int  iq[Nq][4];                  /* array four-site-terms (quad?) */
extern int nn,nw,nq;              /* number of spins and bonds < NN,Nw,Nq */
extern int nu,nd;                       /* number up-/down-spins        */
/* ww-indexed float parameters for wt[],wj[],wu[],we[],wu2,wj2[], etc. */
#define MaxPara 128                     /* max. 8bit=1char      */
extern char numpara;                    /* number of parameters < 128   */
extern double para[MaxPara];            /* Float-Parameters     */


/* index to float parameters para[], moved to tww since v2.20 */
struct tww {                            /* parameter of the Hamiltonien */
 /* one site parameters */
 int wu[NN];                      /* model parameter coulomb  U(i) */
 int we[NN];                      /* model parameter site-energy e(i) */
#if NOSZSYM /* 2017-12 NoSzSym, see below, terms from user wishlist */
 int wfx[NN];			  /* external x-field h(i) per site */
 int wfy[NN];			  /* external y-field h(i) per site = cplx */
#endif
 int wfz[NN];  /* external z-field h(i) per site, 2018-01 f to fz renamed */
 int wd[NN];			  /* single ion anisotropy D(i) */
 /* two site parameters */
 int    w[Nw];                    /* phase-angle (boundary=TBC) */
 double co[Nw],si[Nw];            /* cos(para[w]),sin(para[w])    */
 int    ex[Nw];                   /* 1 bound ok, -1 size exchange */
 int wt[Nw];                      /* model parameter hopping  t(i,j) */
 int wj[Nw];                      /* model parameter exchange J(i,j) */
 int wjz[Nw];             /* extra anisotropy Jz(i,j) xy + xxz-model */
#if NOSZSYM /* 2017-12 NoSzSym, see below, terms from user wishlist */
 int wjx[Nw];                     /* extra anisotropy Jx(i,j) Kitaev */
 int wjy[Nw];                     /* extra anisotropy Jy(i,j) Kitaev */
#endif
 int wv[Nw];                      /* two-site coulomb  V(i,j) ??? */
 int wh[Nw];                      /* hole-hole repulsion */
 /* four site parameters */
 int wq[Nq];                      /* four-site exchange P(i,i+1)+P^-1(i,i+1) */
};    /* ToDo: put local to CPU not global */  

extern struct tww ww;  

#ifndef IFww
#ifdef CONFIG_TBC			/* see config.h */
#define IFww(x)     x
#else
#define IFww(x)
#endif
#endif

#if (0 IFww(+1)) && !defined(CONFIG_TBC)
#error please use CONFIG_TBC instead of IFww in config.h 
#endif

int initvars(const int mpi_i, const char *modelfile); /* mpi_i>0 hide output */
int outmodel();

#endif
